@@ -6,12 +6,14 @@ portfolio, place orders, and get the leaderboard.
 
 Example:
 	>>> from marketwatch import MarketWatch
-	>>> mw = MarketWatch("email", "password")
+    >>> import os
+	>>> mw = MarketWatch(os.environ["MARKETWATCH_USERNAME"], os.environ["MARKETWATCH_PASSWORD"])
 	>>> mw.get_games()
 	>>> mw.get_portfolio(1234)
-	>>> mw.place_order(1234, "AAPL", 10, "buy", "market")
+	>>> mw.buy(1234, "AAPL", 1)
 	>>> mw.get_leaderboard(1234)
 """
+
 import csv
 import json
 
@@ -199,6 +201,16 @@ class MarketWatch:
             ).text.strip()
         )
 
+    # auth decorator in class
+    def auth(func):
+        def wrapper(self, *args, **kwargs):
+            if not self.check_login():
+                self.login()
+            return func(self, *args, **kwargs)
+
+        return wrapper
+
+    @auth
     def get_games(self):
         """
         Get all the games
@@ -207,7 +219,6 @@ class MarketWatch:
 
                         :return: List of games
         """
-        self.check_login()
         games_page = self.session.get("https://www.marketwatch.com/games")
 
         if games_page.status_code != 200:
@@ -247,6 +258,7 @@ class MarketWatch:
             )
         return games_data
 
+    @auth
     def get_game(self, game_id: str) -> list:
         """
         Get a game
@@ -256,7 +268,6 @@ class MarketWatch:
                         :param game_id: Game id
         :return: Game data
         """
-        self.check_login()
         game_page = self.session.get(f"https://www.marketwatch.com/games/{game_id}")
 
         if game_page.status_code != 200:
@@ -345,6 +356,7 @@ class MarketWatch:
             raise MarketWatchException("Failed to get price")
         return f"{ticker.upper()} : ${regions.text}"
 
+    @auth
     def get_portfolio(self, game_id: str):
         """
         Get the portfolio of a game
@@ -354,7 +366,6 @@ class MarketWatch:
 
                         :return: Portfolio of the game
         """
-        self.check_login()
         response = self.session.get(
             f"https://www.marketwatch.com/games/{game_id}/portfolio"
         )
@@ -385,44 +396,21 @@ class MarketWatch:
 
         for row in table:
             cells = row.find_all("td")
+            sign = "-" if cells[4].find("small", {"class": "secondary"}).find("span", {"class": "point"}).text[0] == "-" else "+"
             ticker = cells[1].find("a", {"class": "primary"}).find("mini-quote").text
-            quantity = (
-                cells[1]
-                .find("div", {"class": "secondary"})
-                .find("small", {"class": "text"})
-                .text
-            )
+            quantity = cells[1].find("div", {"class": "secondary"}).find("small").text
             holding = cells[2].find("div", {"class": "secondary"}).text
             holding_percentage = cells[2].find("div", {"class": "primary"}).text
             price = cells[3].find("div", {"class": "primary"}).text
-            price_gain = (
-                cells[3]
-                .find("small", {"class": "secondary"})
-                .find("span", {"class": "point"})
-                .text
-            )
-            price_gain_percentage = (
-                cells[3]
-                .find("small", {"class": "secondary"})
-                .find("span", {"class": "percent"})
-                .text
-            )
+            price_gain = cells[3].find("small", {"class": "secondary"}).find("span", {"class": "point"}).text
+            price_gain_percentage = cells[3].find("small", {"class": "secondary"}).find("span", {"class": "percent"}).text
             value = cells[4].find("div", {"class": "primary"}).text
-            gain = (
-                cells[4]
-                .find("small", {"class": "secondary"})
-                .find("span", {"class": "point"})
-                .text
-            )
-            gain_percentage = (
-                cells[4]
-                .find("small", {"class": "secondary"})
-                .find("span", {"class": "percent"})
-                .text
-            )
+            value_point = cells[4].find("small", {"class": "secondary"}).find("span", {"class": "point"}).text
+            value_percentage = cells[4].find("small", {"class": "secondary"}).find("span", {"class": "percent"}).text
 
             portfolio.append(
                 {
+                    "sign": sign,
                     "ticker": ticker,
                     "quantity": quantity,
                     "holding": holding,
@@ -431,8 +419,8 @@ class MarketWatch:
                     "price_gain": price_gain,
                     "price_gain_percentage": price_gain_percentage,
                     "value": value,
-                    "gain": gain,
-                    "gain_percentage": gain_percentage,
+                    "value_percentage": value_percentage,
+                    "value_point": value_point,
                 }
             )
 
@@ -465,6 +453,121 @@ class MarketWatch:
             "portfolio_allocation": portfolio_allocation,
         }
 
+    @auth
+    def get_portfolio_performance(self, game_id: str, download: bool = False, next_page_url: str = None):
+        """
+        Get the portfolio performance of a game
+
+        :param game_id: Game id
+        :param download: Download the portfolio performance
+        :param next_page_url: Next page url
+
+        :return: Portfolio performance of the game
+        """
+        if next_page_url is None:
+            response = self.session.get(
+                f"https://www.marketwatch.com/games/{game_id}/performance"
+            )
+        else:
+            response = self.session.get(
+                next_page_url
+            )
+
+        if response.status_code != 200:
+            raise MarketWatchException("Game not found")
+
+        soup = BeautifulSoup(response.text, "html.parser")
+
+
+        # ledger_id = self.get_ledger_id(game_id=game_id)
+        ledger_id = soup.find("div", {"class": "element element--table portfolio-performance"})["pub"]
+        table = soup.find("div", {"class": "element element--table portfolio-performance"}).find("tbody").find_all("tr")
+
+        if download:
+            print("Downloaded")
+            return self.session.get(
+                f"https://www.marketwatch.com/games/{game_id}/download?view=performance&amp;pub={ledger_id}&amp;isDownload=true"
+            )
+
+        portfolio_performance = []
+
+        for row in table:
+            cells = row.find_all("td")
+            portfolio_performance.append(
+                {
+                    "date": cells[0].text,
+                    "cash": cells[1].text,
+                    "market_value": cells[2].text,
+                    "total_value": cells[3].text,
+                    "return": cells[4].text,
+                }
+            )
+
+        cursor_next = soup.find("div", {"class": "element element--table portfolio-performance"})["cursor-next"]
+
+        if next_page := soup.find("a", {"class": "link align--right  j-next"}):
+            portfolio_performance += self.get_portfolio_performance(
+                game_id=game_id,
+                download=download,
+                next_page_url=f"https://www.marketwatch.com/games/{game_id}/performance?pub={ledger_id}&cursor={cursor_next}"
+            )
+
+        return portfolio_performance
+
+    @auth
+    def get_transactions(self, game_id: str, download: bool = False, next_page_url: str = None):
+        """
+        Get the transactions of a game
+
+        :param game_id: The game id
+        :param download: Download the transactions as a csv file
+        :param next_page_url: The next page url
+        :return: A list of transactions
+        """
+        response = self.session.get(
+            f"https://www.marketwatch.com/games/{game_id}/transactions"
+        )
+
+        if response.status_code != 200:
+            raise MarketWatchException("Game not found")
+
+        soup = BeautifulSoup(response.text, "html.parser")
+
+        table = soup.find("div", {"class": "element element--table transactions"}).find("tbody").find_all("tr")
+
+        if download:
+            return self.session.get(
+                f"https://www.marketwatch.com/games/{game_id}/download?view=transactions&amp;pub=&amp;isDownload=true"
+            )
+
+        transactions = []
+
+        for row in table:
+            cells = row.find_all("td")
+            transactions.append(
+                {
+                    "symbol": cells[0].text,
+                    "buy_date": cells[1].text,
+                    "sell_date": cells[2].text,
+                    "type": cells[3].text,
+                    "shares": cells[4].text,
+                    "price": cells[5].text,
+                }
+            )
+
+        cursor_next = soup.find("div", {"class": "element element--table portfolio-performance"})["cursor-next"]
+
+        if next_page := soup.find("a", {"class": "link align--right  j-next"}):
+            transactions += self.get_transactions(
+                game_id=game_id,
+                download=download,
+                next_page_url=f"https://www.marketwatch.com/games/{game_id}/transactions?cursor={cursor_next}"
+            )
+
+        return transactions
+
+
+    @auth
     def get_leaderboard(self, game_id: str, download: bool = False):
         """
         Get the leaderboard of a game
@@ -473,7 +576,6 @@ class MarketWatch:
 
         :return: Leaderboard of the game
         """
-        self.check_login()
         if download:
             return self.session.get(
                 f"https://www.marketwatch.com/games/{game_id}/download?view=rankings&amp;pub=&amp;isDownload=true"
@@ -678,6 +780,7 @@ class MarketWatch:
             orderType=OrderType.COVER,
         )
 
+    @auth
     def _create_payload(
         self,
         game_id: str,
@@ -701,7 +804,6 @@ class MarketWatch:
 
         :return: Payload
         """
-        self.check_login()
         ticker_uid = self._get_ticker_uid(ticker)
         response = self.session.post(
             f"https://www.marketwatch.com/games/{game_id}/tradeorder?chartingSymbol={ticker_uid}"
@@ -836,10 +938,16 @@ class MarketWatch:
 
         :param order: Order string
         :return: Price
+
+        Symbol	Shares	% Holdings	Type	Price	Price Change	Price Change %	Value	Value Gain/Loss	Value Gain/Loss %
+        AAPD	12	< 1%	BUY	$24.90	0.27	1.08%	$298.74	$6.66	2.28%
+        AAPL	623	12%	BUY	$156.54	-1.74	-1.10%	$97,524.42	-$1,786.29	-1.80%
+        ANCTF	400	2%	BUY	$47.99	0.19	0.40%	$19,196.00	$374.05	1.99%
+        BAC	100	< 1%	BUY	$28.38	-0.11	-0.38%	$2,838.24	$32.24	1.15%
         """
         return None if ("$" not in order) else float(order[(order.index("$") + 1) :])
 
-    def get_positions(self, game_id: str):
+    def get_positions(self, game_id: str, download: bool = False):
         soup = BeautifulSoup(
             self.session.get(
                 f"https://www.marketwatch.com/games/{game_id}/portfolio"
@@ -847,6 +955,12 @@ class MarketWatch:
             "html.parser",
         )
 
+        if download:
+            return self.session.get(
+                "https://www.marketwatch.com"
+                + soup.select("a[href*='download?view=holdings']")[0]["href"]
+            )
+
         try:
             position_csv = self.session.get(
                 "https://www.marketwatch.com"
@@ -858,13 +972,16 @@ class MarketWatch:
         positions = []
         # extract all lines, skipping the header, in the given csv text
         reader = csv.reader(position_csv.split("\n")[1:])
-        for parts in reader:
-            if len(parts) > 0:
-                avg_entry = float(parts[4].replace("$", "").replace(",", "")) - float(
-                    parts[5]
-                )
-                # create a Position object for each ticker
-                positions.append(Position(parts[0], parts[3], int(parts[1]), avg_entry))
+
+        for row in reader:
+            if len(row) == 0:
+                continue
+
+            ticker = row[0]
+            quantity = int(row[1].replace(",", ""))
+            price = float(row[4].replace("$", "").replace(",", ""))
+            ep = float(row[8].replace("$", "").replace(",", "")) / quantity
+            positions.append(Position(ticker, quantity, price, ep))
 
         return positions
 
@@ -959,7 +1076,7 @@ class MarketWatch:
             text.replace("\r\n", "").replace("\t", "").replace(" ", "").replace(",", "")
         )
 
-    def check_error(self):
+    def check_error_game(self):
         """
         Check if the game is down
 
@@ -969,6 +1086,85 @@ class MarketWatch:
         if self.session.get("https://www.marketwatch.com/games").status_code != 200:
             raise MarketWatchException("Marketwatch Stock Market Game Down")
 
+    # decorator to check if the game is down
+    def check_error(func):
+        def wrapper(*args, **kwargs):
+            args[0].check_error_game()
+            return func(*args, **kwargs)
+        return wrapper
+
+    def create_watchlist(self, name: str):
+        """
+        Create a watchlist
+
+        :param name: Watchlist name
+        :param tickers: List of tickers
+        :return: None
+        """
+        response = self.session.post("https://api.marketwatch.com/api/oskar/me/marketwatch-com")
+
+        if response.status_code != 201:
+            raise MarketWatchException("Failed to create watchlist")
+
+        response_json = response.json()
+
+        return {
+            "Id": response_json["Id"],
+            "Name": name,
+            "TotalItemCount": response_json["TotalItemCount"],
+            "Revision": response_json["Revision"],
+            "Items": response_json["Items"],
+            "CreateDateUtc": response_json["CreateDateUtc"],
+            "LastModifiedDateUtc": response_json["LastModifiedDateUtc"],
+        }
+
+    def add_to_watchlist(self, watchlist_id: str, tickers: list):
+        """
+        Add tickers to a watchlist
+
+        :param watchlist_id: Watchlist ID
+        :param tickers: List of tickers
+        :return: None
+        """
+        items = []
+        for ticker in tickers:
+            self._get_ticker_uid(ticker)
+            items.append({"ChartingSymbol": ticker})
+
+        response = self.session.post(f"https://api.marketwatch.com/api/oskar/me/marketwatch-com/{watchlist_id}/items", json=items)
+
+        if response.status_code != 201:
+            raise MarketWatchException("Failed to add to watchlist")
+        
+        return response.json()
+
+    def get_watchlists(self):
+        """
+        Get all watchlists
+
+        :return: List of watchlists
+        """
+
+        response = self.session.post("https://api.marketwatch.com/api/oskar/me/marketwatch-com/1270679988639624?needed=IndustryClassification|BlueGrassChannels&showItems=true")
+
+        if response.status_code != 200:
+            raise MarketWatchException("Failed to get watchlists")
+
+        return response.json()
+
+    def delete_watchlist_item(self, watchlist_id: str, ticker: str):
+        """
+        Delete a ticker from a watchlist
+
+        :param watchlist_id: Watchlist ID
+        :param ticker: Ticker to delete
+        :return: None
+        """
+        ticker_uid = self._get_ticker_uid(ticker)
+        response = self.session.delete(f"https://api.marketwatch.com/api/oskar/me/marketwatch-com/{watchlist_id}/items/{ticker_uid}")
+
+        if response.status_code != 200:
+            raise MarketWatchException("Failed to delete ticker from watchlist")
 
 if __name__ == "__main__":
     import os
